@@ -27,7 +27,7 @@ class ArgparseCustomFormatter(argparse.HelpFormatter):
         return argparse.HelpFormatter._split_lines(self, text, width)
 
 
-def main():
+def main() -> int:
     # create CLI and parse arguments with argparse
     parser = argparse.ArgumentParser(description=getString("BackuPy: A simple backup program in python with an emphasis on data integrity and transparent behaviour"),
                                      formatter_class=lambda prog: ArgparseCustomFormatter(prog, max_help_position=15),
@@ -75,6 +75,8 @@ def main():
                              "    [compare file attributes and record CRC for changed files]\n"
                              "  CRC\n"
                              "    [compare file attributes and CRC for every file]"))
+    group2.add_argument("--sync-delete", dest="sync_propagate_deletions", action="store_true",
+                        help=getString("Use the database to propagate deletions since the last sync"))
     group2.add_argument("--fi", dest="filter_include_list", action="store", type=str, nargs="+", default=None, metavar="regex",
                         help=getString("Filter: Only include files matching the regular expression(s) (include all by default, searches file paths)"))
     group2.add_argument("--fe", dest="filter_exclude_list", action="store", type=str, nargs="+", default=None, metavar="regex",
@@ -114,10 +116,12 @@ def main():
                         help=getString("Save configuration to <source>/.backupy/config.json"))
     group4.add_argument("-l", "--load", dest="load", action="store_true",
                         help=getString("Load configuration from <source>/.backupy/config.json"))
+    parser.add_argument("--nocolour", dest="nocolour", action="store_true",
+                        help=argparse.SUPPRESS)
     args = parser.parse_args()
     # create and run job
     backup_manager = BackupManager(args)
-    backup_manager.run()
+    return backup_manager.run()
 
 
 if __name__ == "__main__":
